@@ -1,13 +1,20 @@
-/******************************************************************************
-*******************************************************************************
-**
-**  Copyright (C) Sistina Software, Inc.  2002-2003  All rights reserved.
-**  Copyright (C) 2004-2005 Red Hat, Inc. All rights reserved.
-**
-*******************************************************************************
-******************************************************************************/
+/*
+ * Copyright (C) 2002-2003 Sistina Software, Inc. All rights reserved.
+ * Copyright (C) 2004-2009 Red Hat, Inc. All rights reserved.
+ *
+ * This file is part of LVM2.
+ *
+ * This copyrighted material is made available to anyone wishing to use,
+ * modify, copy, or redistribute it subject to the terms and conditions
+ * of the GNU Lesser General Public License v.2.1.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
-/* This provides the interface between clvmd and gulm as the cluster
+/*
+ * This provides the interface between clvmd and gulm as the cluster
  * and lock manager.
  *
  * It also provides the "liblm" functions too as it's hard (and pointless)
@@ -17,7 +24,6 @@
  * on the cluster nodes. That is done in tcp-comms.c
  */
 
-
 #include <pthread.h>
 #include <sys/types.h>
 #include <sys/utsname.h>
@@ -40,18 +46,17 @@
 #include <utmpx.h>
 #include <syslog.h>
 #include <assert.h>
+#include <libdevmapper.h>
+#include <ccs.h>
+#include <libgulm.h>
 
-#include "libdevmapper.h"
-#include "ccs.h"
-#include "list.h"
 #include "locking.h"
-#include "log.h"
+#include "lvm-logging.h"
 #include "clvm.h"
 #include "clvmd-comms.h"
 #include "lvm-functions.h"
 #include "clvmd.h"
 #include "clvmd-gulm.h"
-#include "libgulm.h"
 
 /* Hash list of nodes in the cluster */
 static struct dm_hash_table *node_hash;
@@ -86,12 +91,12 @@ struct lock_wait
 };
 
 /* Forward */
-static int read_from_core_sock(struct local_client *client, char *buf, int len, char *csid,
+static int read_from_core_sock(struct local_client *client, char *buf, int len, const char *csid,
 			       struct local_client **new_client);
-static int read_from_lock_sock(struct local_client *client, char *buf, int len, char *csid,
+static int read_from_lock_sock(struct local_client *client, char *buf, int len, const char *csid,
 			       struct local_client **new_client);
 static int get_all_cluster_nodes(void);
-static int _csid_from_name(char *csid, char *name);
+static int _csid_from_name(char *csid, const char *name);
 static void _cluster_closedown(void);
 
 /* In tcp-comms.c */
@@ -249,7 +254,7 @@ static void _cluster_closedown(void)
 {
     DEBUGLOG("cluster_closedown\n");
     in_shutdown = 1;
-    unlock_all();
+    destroy_lvhash();
     lg_lock_logout(gulm_if);
     lg_core_logout(gulm_if);
     lg_release(gulm_if);
@@ -278,7 +283,7 @@ static void drop_expired_locks(char *nodename)
 }
 
 
-static int read_from_core_sock(struct local_client *client, char *buf, int len, char *csid,
+static int read_from_core_sock(struct local_client *client, char *buf, int len, const char *csid,
 			       struct local_client **new_client)
 {
     int status;
@@ -288,7 +293,7 @@ static int read_from_core_sock(struct local_client *client, char *buf, int len,
     return status<0 ? status : 1;
 }
 
-static int read_from_lock_sock(struct local_client *client, char *buf, int len, char *csid,
+static int read_from_lock_sock(struct local_client *client, char *buf, int len, const char *csid,
 			       struct local_client **new_client)
 {
     int status;
@@ -582,7 +587,7 @@ int get_next_node_csid(void **context, char *csid)
     return 1;
 }
 
-int gulm_name_from_csid(char *csid, char *name)
+int gulm_name_from_csid(const char *csid, char *name)
 {
     struct node_info *ninfo;
 
@@ -598,7 +603,7 @@ int gulm_name_from_csid(char *csid, char *name)
 }
 
 
-static int _csid_from_name(char *csid, char *name)
+static int _csid_from_name(char *csid, const char *name)
 {
     struct dm_hash_node *hn;
     struct node_info *ninfo;
@@ -622,7 +627,7 @@ static int _get_num_nodes()
 }
 
 /* Node is now known to be running a clvmd */
-void gulm_add_up_node(char *csid)
+void gulm_add_up_node(const char *csid)
 {
     struct node_info *ninfo;
 
@@ -661,10 +666,11 @@ void add_down_node(char *csid)
 
 /* Call a callback for each node, so the caller knows whether it's up or down */
 static int _cluster_do_node_callback(struct local_client *master_client,
-				     void (*callback)(struct local_client *, char *csid, int node_up))
+				     void (*callback)(struct local_client *, const char *csid, int node_up))
 {
     struct dm_hash_node *hn;
     struct node_info *ninfo;
+    int somedown = 0;
 
     dm_hash_iterate(hn, node_hash)
     {
@@ -686,12 +692,14 @@ static int _cluster_do_node_callback(struct local_client *master_client,
 	    client = dm_hash_lookup_binary(sock_hash, csid, GULM_MAX_CSID_LEN);
 
 	}
+	DEBUGLOG("down_callback2. node %s, state = %d\n", ninfo->name, ninfo->state);
 	if (ninfo->state != NODE_DOWN)
 		callback(master_client, csid, ninfo->state == NODE_CLVMD);
 
-
+	if (ninfo->state != NODE_CLVMD)
+		somedown = -1;
     }
-    return 0;
+    return somedown;
 }
 
 /* Convert gulm error codes to unix errno numbers */
@@ -730,7 +738,7 @@ static int _lock_resource(char *resource, int mode, int flags, int *lockid)
     pthread_mutex_lock(&lwait.mutex);
 
     /* This needs to be converted from DLM/LVM2 value for GULM */
-    if (flags == LCK_NONBLOCK) flags = lg_lock_flag_Try;
+    if (flags & LKF_NOQUEUE) flags = lg_lock_flag_Try;
 
     dm_hash_insert(lock_hash, resource, &lwait);
     DEBUGLOG("lock_resource '%s', flags=%d, mode=%d\n", resource, flags, mode);
@@ -828,6 +836,7 @@ static int _sync_lock(const char *resource, int mode, int flags, int *lockid)
         }
 	break;
 
+    case LCK_PREAD:
     case LCK_READ:
 	status = _lock_resource(lock1, lg_lock_state_Shared, flags, lockid);
 	if (status)
@@ -864,6 +873,7 @@ static int _sync_unlock(const char *resource, int lockid)
     /* The held lock mode is in the lock id */
     assert(lockid == LCK_EXCL ||
 	   lockid == LCK_READ ||
+	   lockid == LCK_PREAD ||
 	   lockid == LCK_WRITE);
 
     status = _unlock_resource(lock1, lockid);
@@ -946,7 +956,7 @@ static int get_all_cluster_nodes()
 		}
 		else {
 			DEBUGLOG("Cannot resolve host name %s\n", nodename);
-			log_err("Cannot resolve host name %s\n", nodename);
+			log_error("Cannot resolve host name %s\n", nodename);
 		}
 	}
 	free(nodename);
@@ -963,14 +973,20 @@ static int _get_main_cluster_fd(void)
 	return get_main_gulm_cluster_fd();
 }
 
-static int _cluster_fd_callback(struct local_client *fd, char *buf, int len, char *csid, struct local_client **new_client)
+static int _cluster_fd_callback(struct local_client *fd, char *buf, int len, const char *csid, struct local_client **new_client)
 {
 	return cluster_fd_gulm_callback(fd, buf, len, csid, new_client);
 }
 
-static int _cluster_send_message(void *buf, int msglen, char *csid, const char *errtext)
+static int _cluster_send_message(const void *buf, int msglen, const char *csid, const char *errtext)
 {
-	return gulm_cluster_send_message(buf, msglen, csid, errtext);
+	return gulm_cluster_send_message((char *)buf, msglen, csid, errtext);
+}
+
+static int _get_cluster_name(char *buf, int buflen)
+{
+	strncpy(buf, cluster_name, buflen);
+	return 0;
 }
 
 static struct cluster_ops _cluster_gulm_ops = {
@@ -987,6 +1003,7 @@ static struct cluster_ops _cluster_gulm_ops = {
 	.add_up_node              = gulm_add_up_node,
 	.reread_config            = _reread_config,
 	.cluster_closedown        = _cluster_closedown,
+	.get_cluster_name         = _get_cluster_name,
 	.sync_lock                = _sync_lock,
 	.sync_unlock              = _sync_unlock,
 };
