@@ -135,6 +135,53 @@ uint32_t adjusted_mirror_region_size(uint32_t extent_size, uint32_t extents,
 	return region_size;
 }
 
+/*
+ * shift_mirror_images
+ * @mirrored_seg
+ * @mimage:  The position (index) of the image to move to the end
+ *
+ * When dealing with removal of legs, we often move a 'removable leg'
+ * to the back of the 'areas' array.  It is critically important not
+ * to simply swap it for the last area in the array.  This would have
+ * the affect of reordering the remaining legs - altering position of
+ * the primary.  So, we must shuffle all of the areas in the array
+ * to maintain their relative position before moving the 'removable
+ * leg' to the end.
+ *
+ * Short illustration of the problem:
+ *   - Mirror consists of legs A, B, C and we want to remove A
+ *   - We swap A and C and then remove A, leaving C, B
+ * This scenario is problematic in failure cases where A dies, because
+ * B becomes the primary.  If the above happens, we effectively throw
+ * away any changes made between the time of failure and the time of
+ * restructuring the mirror.
+ *
+ * So, any time we want to move areas to the end to be removed, use
+ * this function.
+ */
+int shift_mirror_images(struct lv_segment *mirrored_seg, unsigned mimage)
+{
+	int i;
+	struct lv_segment_area area;
+
+	if (mimage >= mirrored_seg->area_count) {
+		log_error("Invalid index (%u) of mirror image supplied "
+			  "to shift_mirror_images()", mimage);
+		return 0;
+	}
+
+	area = mirrored_seg->areas[mimage];
+
+	/* Shift remaining images down to fill the hole */
+	for (i = mimage + 1; i < mirrored_seg->area_count; i++)
+		mirrored_seg->areas[i-1] = mirrored_seg->areas[i];
+
+	/* Place this one at the end */
+	mirrored_seg->areas[i-1] = area;
+
+	return 1;
+}
+
 /*
  * This function writes a new header to the mirror log header to the lv
  *
@@ -192,7 +239,7 @@ static int _write_log_header(struct cmd_context *cmd, struct logical_volume *lv)
  */
 static int _init_mirror_log(struct cmd_context *cmd,
 			    struct logical_volume *log_lv, int in_sync,
-			    struct list *tags, int remove_on_failure)
+			    struct dm_list *tags, int remove_on_failure)
 {
 	struct str_list *sl;
 	struct lvinfo info;
@@ -213,31 +260,28 @@ static int _init_mirror_log(struct cmd_context *cmd,
 	}
 
 	/* Temporary make it visible for set_lv() */
-	log_lv->status |= VISIBLE_LV;
+	lv_set_visible(log_lv);
 
 	/* Temporary tag mirror log for activation */
-	list_iterate_items(sl, tags)
+	dm_list_iterate_items(sl, tags)
 		if (!str_list_add(cmd->mem, &log_lv->tags, sl->str)) {
 			log_error("Aborting. Unable to tag mirror log.");
 			goto activate_lv;
 		}
 
 	/* store mirror log on disk(s) */
-	if (!vg_write(log_lv->vg))
+	if (!vg_write(log_lv->vg) || !vg_commit(log_lv->vg))
 		goto activate_lv;
 
 	backup(log_lv->vg);
 
-	if (!vg_commit(log_lv->vg))
-		goto activate_lv;
-
 	if (!activate_lv(cmd, log_lv)) {
 		log_error("Aborting. Failed to activate mirror log.");
 		goto revert_new_lv;
 	}
 
 	/* Remove the temporary tags */
-	list_iterate_items(sl, tags)
+	dm_list_iterate_items(sl, tags)
 		if (!str_list_del(&log_lv->tags, sl->str))
 			log_error("Failed to remove tag %s from mirror log.",
 				  sl->str);
@@ -259,7 +303,7 @@ static int _init_mirror_log(struct cmd_context *cmd,
 		return 0;
 	}
 
-	log_lv->status &= ~VISIBLE_LV;
+	lv_set_hidden(log_lv);
 
 	if (was_active && !activate_lv(cmd, log_lv))
 		return_0;
@@ -276,7 +320,7 @@ deactivate_and_revert_new_lv:
 revert_new_lv:
 	log_lv->status = orig_status;
 
-	list_iterate_items(sl, tags)
+	dm_list_iterate_items(sl, tags)
 		if (!str_list_del(&log_lv->tags, sl->str))
 			log_error("Failed to remove tag %s from mirror log.",
 				  sl->str);
@@ -287,10 +331,12 @@ revert_new_lv:
 		return 0;
 	}
 
-	if (!vg_write(log_lv->vg) ||
-	    (backup(log_lv->vg), !vg_commit(log_lv->vg)))
+	if (!vg_write(log_lv->vg) || !vg_commit(log_lv->vg))
 		log_error("Manual intervention may be required to "
 			  "remove/restore abandoned log LV before retrying.");
+	else
+		backup(log_lv->vg);
+
 activate_lv:
 	if (was_active && !remove_on_failure && !activate_lv(cmd, log_lv))
 		return_0;
@@ -308,7 +354,7 @@ static int _delete_lv(struct logical_volume *mirror_lv, struct logical_volume *l
 
 	/* Inherit tags - maybe needed for activation */
 	if (!str_list_match_list(&mirror_lv->tags, &lv->tags)) {
-		list_iterate_items(sl, &mirror_lv->tags)
+		dm_list_iterate_items(sl, &mirror_lv->tags)
 			if (!str_list_add(cmd->mem, &lv->tags, sl->str)) {
 				log_error("Aborting. Unable to tag.");
 				return 0;
@@ -334,9 +380,9 @@ static int _delete_lv(struct logical_volume *mirror_lv, struct logical_volume *l
 }
 
 static int _merge_mirror_images(struct logical_volume *lv,
-				const struct list *mimages)
+				const struct dm_list *mimages)
 {
-	uint32_t addition = list_size(mimages);
+	uint32_t addition = dm_list_size(mimages);
 	struct logical_volume **img_lvs;
 	struct lv_list *lvl;
 	int i = 0;
@@ -347,7 +393,7 @@ static int _merge_mirror_images(struct logical_volume *lv,
 	if (!(img_lvs = alloca(sizeof(*img_lvs) * addition)))
 		return_0;
 
-	list_iterate_items(lvl, mimages)
+	dm_list_iterate_items(lvl, mimages)
 		img_lvs[i++] = lvl->lv;
 
 	return lv_add_mirror_lvs(lv, img_lvs, addition,
@@ -364,7 +410,7 @@ struct logical_volume *detach_mirror_log(struct lv_segment *mirrored_seg)
 
 	log_lv = mirrored_seg->log_lv;
 	mirrored_seg->log_lv = NULL;
-	log_lv->status |= VISIBLE_LV;
+	lv_set_visible(log_lv);
 	log_lv->status &= ~MIRROR_LOG;
 	remove_seg_from_segs_using_this_lv(log_lv, mirrored_seg);
 
@@ -373,7 +419,7 @@ struct logical_volume *detach_mirror_log(struct lv_segment *mirrored_seg)
 
 /* Check if mirror image LV is removable with regard to given removable_pvs */
 static int _is_mirror_image_removable(struct logical_volume *mimage_lv,
-				      struct list *removable_pvs)
+				      struct dm_list *removable_pvs)
 {
 	struct physical_volume *pv;
 	struct lv_segment *seg;
@@ -381,7 +427,7 @@ static int _is_mirror_image_removable(struct logical_volume *mimage_lv,
 	struct pv_list *pvl;
 	uint32_t s;
 
-	list_iterate_items(seg, &mimage_lv->segments) {
+	dm_list_iterate_items(seg, &mimage_lv->segments) {
 		for (s = 0; s < seg->area_count; s++) {
 			if (seg_type(seg, s) != AREA_PV) {
 				/* FIXME Recurse for AREA_LV? */
@@ -393,8 +439,13 @@ static int _is_mirror_image_removable(struct logical_volume *mimage_lv,
 			pv = seg_pv(seg, s);
 
 			pv_found = 0;
-			list_iterate_items(pvl, removable_pvs) {
-				if (pv->dev->dev == pvl->pv->dev->dev) {
+			dm_list_iterate_items(pvl, removable_pvs) {
+				if (id_equal(&pv->id, &pvl->pv->id)) {
+					pv_found = 1;
+					break;
+				}
+				if (pvl->pv->dev && pv->dev &&
+				    pv->dev->dev == pvl->pv->dev->dev) {
 					pv_found = 1;
 					break;
 				}
@@ -434,7 +485,7 @@ static int _is_mirror_image_removable(struct logical_volume *mimage_lv,
  */
 static int _remove_mirror_images(struct logical_volume *lv,
 				 uint32_t num_removed,
-				 struct list *removable_pvs,
+				 struct dm_list *removable_pvs,
 				 unsigned remove_log, unsigned collapse,
 				 uint32_t *removed)
 {
@@ -444,11 +495,10 @@ static int _remove_mirror_images(struct logical_volume *lv,
 	struct logical_volume *detached_log_lv = NULL;
 	struct logical_volume *lv1 = NULL;
 	struct lv_segment *mirrored_seg = first_seg(lv);
-	struct lv_segment_area area;
 	uint32_t old_area_count = mirrored_seg->area_count;
 	uint32_t new_area_count = mirrored_seg->area_count;
 	struct lv_list *lvl;
-	struct list tmp_orphan_lvs;
+	struct dm_list tmp_orphan_lvs;
 
 	if (removed)
 		*removed = 0;
@@ -469,13 +519,12 @@ static int _remove_mirror_images(struct logical_volume *lv,
 		for (s = 0; s < mirrored_seg->area_count &&
 			    old_area_count - new_area_count < num_removed; s++) {
 			sub_lv = seg_lv(mirrored_seg, s);
+
 			if (!is_temporary_mirror_layer(sub_lv) &&
 			    _is_mirror_image_removable(sub_lv, removable_pvs)) {
-				/* Swap segment to end */
+				if (!shift_mirror_images(mirrored_seg, s))
+					return_0;
 				new_area_count--;
-				area = mirrored_seg->areas[new_area_count];
-				mirrored_seg->areas[new_area_count] = mirrored_seg->areas[s];
-				mirrored_seg->areas[s] = area;
 			}
 		}
 		if (num_removed && old_area_count == new_area_count)
@@ -484,16 +533,16 @@ static int _remove_mirror_images(struct logical_volume *lv,
 		new_area_count = old_area_count - num_removed;
 
 	/* Remove mimage LVs from the segment */
-	list_init(&tmp_orphan_lvs);
+	dm_list_init(&tmp_orphan_lvs);
 	for (m = new_area_count; m < mirrored_seg->area_count; m++) {
 		seg_lv(mirrored_seg, m)->status &= ~MIRROR_IMAGE;
-		seg_lv(mirrored_seg, m)->status |= VISIBLE_LV;
+		lv_set_visible(seg_lv(mirrored_seg, m));
 		if (!(lvl = dm_pool_alloc(lv->vg->cmd->mem, sizeof(*lvl)))) {
 			log_error("lv_list alloc failed");
 			return 0;
 		}
 		lvl->lv = seg_lv(mirrored_seg, m);
-		list_add(&tmp_orphan_lvs, &lvl->list);
+		dm_list_add(&tmp_orphan_lvs, &lvl->list);
 		release_lv_segment_area(mirrored_seg, m, mirrored_seg->area_len);
 	}
 	mirrored_seg->area_count = new_area_count;
@@ -505,7 +554,7 @@ static int _remove_mirror_images(struct logical_volume *lv,
 	if (new_area_count == 1 && !is_temporary_mirror_layer(lv)) {
 		lv1 = seg_lv(mirrored_seg, 0);
 		lv1->status &= ~MIRROR_IMAGE;
-		lv1->status |= VISIBLE_LV;
+		lv_set_visible(lv1);
 		detached_log_lv = detach_mirror_log(mirrored_seg);
 		if (!remove_layer_from_lv(lv, lv1))
 			return_0;
@@ -552,6 +601,17 @@ static int _remove_mirror_images(struct logical_volume *lv,
 
 	log_very_verbose("Updating \"%s\" in kernel", mirrored_seg->lv->name);
 
+	/*
+	 * Avoid having same mirror target loaded twice simultaneously by first
+	 * resuming the removed LV which now contains an error segment.
+	 * As it's now detached from mirrored_seg->lv we must resume it
+	 * explicitly.
+	 */
+	if (lv1 && !resume_lv(lv1->vg->cmd, lv1)) {
+		log_error("Problem resuming temporary LV, %s", lv1->name);
+		return 0;
+	}
+
 	if (!resume_lv(mirrored_seg->lv->vg->cmd, mirrored_seg->lv)) {
 		log_error("Problem reactivating %s", mirrored_seg->lv->name);
 		return 0;
@@ -559,7 +619,7 @@ static int _remove_mirror_images(struct logical_volume *lv,
 
 	/* Save or delete the 'orphan' LVs */
 	if (!collapse) {
-		list_iterate_items(lvl, &tmp_orphan_lvs)
+		dm_list_iterate_items(lvl, &tmp_orphan_lvs)
 			if (!_delete_lv(lv, lvl->lv))
 				return_0;
 	}
@@ -595,7 +655,7 @@ static int _remove_mirror_images(struct logical_volume *lv,
  * Remove the number of mirror images from the LV
  */
 int remove_mirror_images(struct logical_volume *lv, uint32_t num_mirrors,
-			 struct list *removable_pvs, unsigned remove_log)
+			 struct dm_list *removable_pvs, unsigned remove_log)
 {
 	uint32_t num_removed, removed_once, r;
 	uint32_t existing_mirrors = lv_mirror_count(lv);
@@ -641,17 +701,16 @@ int remove_mirror_images(struct logical_volume *lv, uint32_t num_mirrors,
 static int _mirrored_lv_in_sync(struct logical_volume *lv)
 {
 	float sync_percent;
+	percent_range_t percent_range;
 
-	if (!lv_mirror_percent(lv->vg->cmd, lv, 0, &sync_percent, NULL)) {
+	if (!lv_mirror_percent(lv->vg->cmd, lv, 0, &sync_percent,
+			       &percent_range, NULL)) {
 		log_error("Unable to determine mirror sync status of %s/%s.",
 			  lv->vg->name, lv->name);
 		return 0;
 	}
 
-	if (sync_percent >= 100.0)
-		return 1;
-
-	return 0;
+	return (percent_range == PERCENT_100) ? 1 : 0;
 }
 
 /*
@@ -788,7 +847,7 @@ static int replace_mirror_images(struct lv_segment *mirrored_seg,
 }
 
 int reconfigure_mirror_images(struct lv_segment *mirrored_seg, uint32_t num_mirrors,
-			      struct list *removable_pvs, unsigned remove_log)
+			      struct dm_list *removable_pvs, unsigned remove_log)
 {
 	int r;
 	int in_sync;
@@ -872,7 +931,7 @@ static int _create_mimage_lvs(struct alloc_handle *ah,
 	for (m = 0; m < num_mirrors; m++) {
 		if (!(img_lvs[m] = lv_create_empty(img_name,
 					     NULL, LVM_READ | LVM_WRITE,
-					     ALLOC_INHERIT, 0, lv->vg))) {
+					     ALLOC_INHERIT, lv->vg))) {
 			log_error("Aborting. Failed to create mirror image LV. "
 				  "Remove new LV and retry.");
 			return 0;
@@ -905,7 +964,7 @@ int remove_mirrors_from_segments(struct logical_volume *lv,
 	uint32_t s;
 
 	/* Check the segment params are compatible */
-	list_iterate_items(seg, &lv->segments) {
+	dm_list_iterate_items(seg, &lv->segments) {
 		if (!seg_is_mirrored(seg)) {
 			log_error("Segment is not mirrored: %s:%" PRIu32,
 				  lv->name, seg->le);
@@ -919,7 +978,7 @@ int remove_mirrors_from_segments(struct logical_volume *lv,
 	}
 
 	/* Convert the segments */
-	list_iterate_items(seg, &lv->segments) {
+	dm_list_iterate_items(seg, &lv->segments) {
 		if (!new_mirrors && seg->extents_copied == seg->area_len) {
 			if (!move_lv_segment_area(seg, 0, seg, 1))
 				return_0;
@@ -942,7 +1001,7 @@ const char *get_pvmove_pvname_from_lv_mirr(struct logical_volume *lv_mirr)
 {
 	struct lv_segment *seg;
 
-	list_iterate_items(seg, &lv_mirr->segments) {
+	dm_list_iterate_items(seg, &lv_mirr->segments) {
 		if (!seg_is_mirrored(seg))
 			continue;
 		if (seg_type(seg, 0) != AREA_PV)
@@ -958,7 +1017,7 @@ const char *get_pvmove_pvname_from_lv(struct logical_volume *lv)
 	struct lv_segment *seg;
 	uint32_t s;
 
-	list_iterate_items(seg, &lv->segments) {
+	dm_list_iterate_items(seg, &lv->segments) {
 		for (s = 0; s < seg->area_count; s++) {
 			if (seg_type(seg, s) != AREA_LV)
 				continue;
@@ -978,14 +1037,14 @@ struct logical_volume *find_pvmove_lv(struct volume_group *vg,
 	struct lv_segment *seg;
 
 	/* Loop through all LVs */
-	list_iterate_items(lvl, &vg->lvs) {
+	dm_list_iterate_items(lvl, &vg->lvs) {
 		lv = lvl->lv;
 
 		if (!(lv->status & lv_type))
 			continue;
 
 		/* Check segment origins point to pvname */
-		list_iterate_items(seg, &lv->segments) {
+		dm_list_iterate_items(seg, &lv->segments) {
 			if (seg_type(seg, 0) != AREA_PV)
 				continue;
 			if (seg_dev(seg, 0) != dev)
@@ -998,9 +1057,10 @@ struct logical_volume *find_pvmove_lv(struct volume_group *vg,
 }
 
 struct logical_volume *find_pvmove_lv_from_pvname(struct cmd_context *cmd,
-					 	  struct volume_group *vg,
-				      		  const char *name,
-				      		  uint32_t lv_type)
+						  struct volume_group *vg,
+						  const char *name,
+						  const char *uuid __attribute((unused)),
+						  uint32_t lv_type)
 {
 	struct physical_volume *pv;
 
@@ -1010,10 +1070,10 @@ struct logical_volume *find_pvmove_lv_from_pvname(struct cmd_context *cmd,
 	return find_pvmove_lv(vg, pv->dev, lv_type);
 }
 
-struct list *lvs_using_lv(struct cmd_context *cmd, struct volume_group *vg,
+struct dm_list *lvs_using_lv(struct cmd_context *cmd, struct volume_group *vg,
 			  struct logical_volume *lv)
 {
-	struct list *lvs;
+	struct dm_list *lvs;
 	struct logical_volume *lv1;
 	struct lv_list *lvl, *lvl1;
 	struct lv_segment *seg;
@@ -1024,16 +1084,16 @@ struct list *lvs_using_lv(struct cmd_context *cmd, struct volume_group *vg,
 		return NULL;
 	}
 
-	list_init(lvs);
+	dm_list_init(lvs);
 
 	/* Loop through all LVs except the one supplied */
-	list_iterate_items(lvl1, &vg->lvs) {
+	dm_list_iterate_items(lvl1, &vg->lvs) {
 		lv1 = lvl1->lv;
 		if (lv1 == lv)
 			continue;
 
 		/* Find whether any segment points at the supplied LV */
-		list_iterate_items(seg, &lv1->segments) {
+		dm_list_iterate_items(seg, &lv1->segments) {
 			for (s = 0; s < seg->area_count; s++) {
 				if (seg_type(seg, s) != AREA_LV ||
 				    seg_lv(seg, s) != lv)
@@ -1043,7 +1103,7 @@ struct list *lvs_using_lv(struct cmd_context *cmd, struct volume_group *vg,
 					return NULL;
 				}
 				lvl->lv = lv1;
-				list_add(lvs, &lvl->list);
+				dm_list_add(lvs, &lvl->list);
 				goto next_lv;
 			}
 		}
@@ -1054,12 +1114,13 @@ struct list *lvs_using_lv(struct cmd_context *cmd, struct volume_group *vg,
 	return lvs;
 }
 
-float copy_percent(struct logical_volume *lv_mirr)
+float copy_percent(struct logical_volume *lv_mirr,
+		   percent_range_t *percent_range)
 {
 	uint32_t numerator = 0u, denominator = 0u;
 	struct lv_segment *seg;
 
-	list_iterate_items(seg, &lv_mirr->segments) {
+	dm_list_iterate_items(seg, &lv_mirr->segments) {
 		denominator += seg->area_len;
 
 		if (seg_is_mirrored(seg) && seg->area_count > 1)
@@ -1068,6 +1129,13 @@ float copy_percent(struct logical_volume *lv_mirr)
 			numerator += seg->area_len;
 	}
 
+	if (!denominator || (numerator == denominator))
+		*percent_range = PERCENT_100;
+	else if (numerator == 0)
+		*percent_range = PERCENT_0;
+	else
+		*percent_range = PERCENT_0_TO_100;
+		
 	return denominator ? (float) numerator *100 / denominator : 100.0;
 }
 
@@ -1079,8 +1147,8 @@ int fixup_imported_mirrors(struct volume_group *vg)
 	struct lv_list *lvl;
 	struct lv_segment *seg;
 
-	list_iterate_items(lvl, &vg->lvs) {
-		list_iterate_items(seg, &lvl->lv->segments) {
+	dm_list_iterate_items(lvl, &vg->lvs) {
+		dm_list_iterate_items(seg, &lvl->lv->segments) {
 			if (seg->segtype !=
 			    get_segtype_from_string(vg->cmd, "mirror"))
 				continue;
@@ -1098,12 +1166,13 @@ int fixup_imported_mirrors(struct volume_group *vg)
  */
 int add_mirrors_to_segments(struct cmd_context *cmd, struct logical_volume *lv,
 			    uint32_t mirrors, uint32_t region_size,
-			    struct list *allocatable_pvs, alloc_policy_t alloc)
+			    struct dm_list *allocatable_pvs, alloc_policy_t alloc)
 {
 	struct alloc_handle *ah;
 	const struct segment_type *segtype;
-	struct list *parallel_areas;
+	struct dm_list *parallel_areas;
 	uint32_t adjusted_region_size;
+	int r = 1;
 
 	if (!(parallel_areas = build_parallel_areas_from_lv(cmd, lv)))
 		return_0;
@@ -1115,7 +1184,7 @@ int add_mirrors_to_segments(struct cmd_context *cmd, struct logical_volume *lv,
 							   lv->le_count,
 							   region_size);
 
-	if (!(ah = allocate_extents(lv->vg, NULL, segtype, 1, mirrors, 0,
+	if (!(ah = allocate_extents(lv->vg, NULL, segtype, 1, mirrors, 0, 0,
 				    lv->le_count, allocatable_pvs, alloc,
 				    parallel_areas))) {
 		log_error("Unable to allocate mirror extents for %s.", lv->name);
@@ -1124,10 +1193,11 @@ int add_mirrors_to_segments(struct cmd_context *cmd, struct logical_volume *lv,
 
 	if (!lv_add_mirror_areas(ah, lv, 0, adjusted_region_size)) {
 		log_error("Failed to add mirror areas to %s", lv->name);
-		return 0;
+		r = 0;
 	}
 
-	return 1;
+	alloc_destroy(ah);
+	return r;
 }
 
 /*
@@ -1137,36 +1207,38 @@ int add_mirrors_to_segments(struct cmd_context *cmd, struct logical_volume *lv,
  */
 int remove_mirror_log(struct cmd_context *cmd,
 		      struct logical_volume *lv,
-		      struct list *removable_pvs)
+		      struct dm_list *removable_pvs)
 {
 	float sync_percent;
+	percent_range_t percent_range = PERCENT_0;
 	struct lvinfo info;
 	struct volume_group *vg = lv->vg;
 
 	/* Unimplemented features */
-	if (list_size(&lv->segments) != 1) {
+	if (dm_list_size(&lv->segments) != 1) {
 		log_error("Multiple-segment mirror is not supported");
 		return 0;
 	}
 
 	/* Had disk log, switch to core. */
 	if (lv_info(cmd, lv, &info, 0, 0) && info.exists) {
-		if (!lv_mirror_percent(cmd, lv, 0, &sync_percent, NULL)) {
+		if (!lv_mirror_percent(cmd, lv, 0, &sync_percent,
+				       &percent_range, NULL)) {
 			log_error("Unable to determine mirror sync status.");
 			return 0;
 		}
 	} else if (vg_is_clustered(vg)) {
-		log_error("Unable to convert the log of inactive "
-			  "cluster mirror %s", lv->name);
+		log_error("Unable to convert the log of an inactive "
+			  "cluster mirror, %s", lv->name);
 		return 0;
 	} else if (yes_no_prompt("Full resync required to convert "
 				 "inactive mirror %s to core log. "
-				 "Proceed? [y/n]: "))
+				 "Proceed? [y/n]: ") == 'y')
 		sync_percent = 0;
 	else
 		return 0;
 
-	if (sync_percent >= 100.0)
+	if (percent_range == PERCENT_100)
 		init_mirror_in_sync(1);
 	else {
 		/* A full resync will take place */
@@ -1204,7 +1276,7 @@ static struct logical_volume *_create_mirror_log(struct logical_volume *lv,
 
 	if (!(log_lv = lv_create_empty(log_name, NULL,
 				       VISIBLE_LV | LVM_READ | LVM_WRITE,
-				       alloc, 0, lv->vg)))
+				       alloc, lv->vg)))
 		return_NULL;
 
 	if (!lv_add_log_segment(ah, log_lv))
@@ -1278,21 +1350,23 @@ int attach_mirror_log(struct lv_segment *seg, struct logical_volume *log_lv)
 {
 	seg->log_lv = log_lv;
 	log_lv->status |= MIRROR_LOG;
-	log_lv->status &= ~VISIBLE_LV;
+	lv_set_hidden(log_lv);
 	return add_seg_to_segs_using_this_lv(log_lv, seg);
 }
 
 int add_mirror_log(struct cmd_context *cmd, struct logical_volume *lv,
 		   uint32_t log_count, uint32_t region_size,
-		   struct list *allocatable_pvs, alloc_policy_t alloc)
+		   struct dm_list *allocatable_pvs, alloc_policy_t alloc)
 {
 	struct alloc_handle *ah;
 	const struct segment_type *segtype;
-	struct list *parallel_areas;
+	struct dm_list *parallel_areas;
 	float sync_percent;
+	percent_range_t percent_range;
 	int in_sync;
 	struct logical_volume *log_lv;
 	struct lvinfo info;
+	int r = 0;
 
 	/* Unimplemented features */
 	if (log_count > 1) {
@@ -1300,7 +1374,7 @@ int add_mirror_log(struct cmd_context *cmd, struct logical_volume *lv,
 		return 0;
 	}
 
-	if (list_size(&lv->segments) != 1) {
+	if (dm_list_size(&lv->segments) != 1) {
 		log_error("Multiple-segment mirror is not supported");
 		return 0;
 	}
@@ -1324,7 +1398,7 @@ int add_mirror_log(struct cmd_context *cmd, struct logical_volume *lv,
 		return_0;
 
 	if (activation() && segtype->ops->target_present &&
-	    !segtype->ops->target_present(NULL, NULL)) {
+	    !segtype->ops->target_present(cmd, NULL, NULL)) {
 		log_error("%s: Required device-mapper target(s) not "
 			  "detected in your kernel", segtype->name);
 		return 0;
@@ -1332,7 +1406,7 @@ int add_mirror_log(struct cmd_context *cmd, struct logical_volume *lv,
 
 	/* allocate destination extents */
 	ah = allocate_extents(lv->vg, NULL, segtype,
-			      0, 0, log_count, 0,
+			      0, 0, log_count, region_size, 0,
 			      allocatable_pvs, alloc, parallel_areas);
 	if (!ah) {
 		log_error("Unable to allocate extents for mirror log.");
@@ -1340,21 +1414,24 @@ int add_mirror_log(struct cmd_context *cmd, struct logical_volume *lv,
 	}
 
 	/* check sync status */
-	if (lv_mirror_percent(cmd, lv, 0, &sync_percent, NULL) &&
-	    sync_percent >= 100.0)
+	if (lv_mirror_percent(cmd, lv, 0, &sync_percent, &percent_range,
+			      NULL) &&
+	    (percent_range == PERCENT_100))
 		in_sync = 1;
 	else
 		in_sync = 0;
 
 	if (!(log_lv = _set_up_mirror_log(cmd, ah, lv, log_count,
 					  region_size, alloc, in_sync)))
-		return_0;
+		goto_out;
 
 	if (!attach_mirror_log(first_seg(lv), log_lv))
-		return_0;
+		goto_out;
 
+	r = 1;
+out:
 	alloc_destroy(ah);
-	return 1;
+	return r;
 }
 
 /*
@@ -1362,12 +1439,12 @@ int add_mirror_log(struct cmd_context *cmd, struct logical_volume *lv,
  */
 int add_mirror_images(struct cmd_context *cmd, struct logical_volume *lv,
 		      uint32_t mirrors, uint32_t stripes, uint32_t region_size,
-		      struct list *allocatable_pvs, alloc_policy_t alloc,
+		      struct dm_list *allocatable_pvs, alloc_policy_t alloc,
 		      uint32_t log_count)
 {
 	struct alloc_handle *ah;
 	const struct segment_type *segtype;
-	struct list *parallel_areas;
+	struct dm_list *parallel_areas;
 	struct logical_volume **img_lvs;
 	struct logical_volume *log_lv = NULL;
 
@@ -1387,7 +1464,7 @@ int add_mirror_images(struct cmd_context *cmd, struct logical_volume *lv,
 		return_0;
 
 	ah = allocate_extents(lv->vg, NULL, segtype,
-			      stripes, mirrors, log_count, lv->le_count,
+			      stripes, mirrors, log_count, region_size, lv->le_count,
 			      allocatable_pvs, alloc, parallel_areas);
 	if (!ah) {
 		log_error("Unable to allocate extents for mirror(s).");
@@ -1399,8 +1476,10 @@ int add_mirror_images(struct cmd_context *cmd, struct logical_volume *lv,
 	 */
 	if (log_count &&
 	    !(log_lv = _set_up_mirror_log(cmd, ah, lv, log_count, region_size,
-					  alloc, mirror_in_sync())))
-		return_0;
+					  alloc, mirror_in_sync()))) {
+		stack;
+		goto out_remove_images;
+	}
 
 	/* The log initialization involves vg metadata commit.
 	   So from here on, if failure occurs, the log must be explicitly
@@ -1409,7 +1488,7 @@ int add_mirror_images(struct cmd_context *cmd, struct logical_volume *lv,
 	/*
 	 * insert a mirror layer
 	 */
-	if (list_size(&lv->segments) != 1 ||
+	if (dm_list_size(&lv->segments) != 1 ||
 	    seg_type(first_seg(lv), 0) != AREA_LV)
 		if (!insert_layer_for_lv(cmd, lv, 0, "_mimage_%d"))
 			goto out_remove_log;
@@ -1431,7 +1510,7 @@ int add_mirror_images(struct cmd_context *cmd, struct logical_volume *lv,
 			       region_size)) {
 		log_error("Aborting. Failed to add mirror segment. "
 			  "Remove new LV and retry.");
-		goto out_remove_imgs;
+		goto out_remove_images;
 	}
 
 	if (log_count && !attach_mirror_log(first_seg(lv), log_lv))
@@ -1441,12 +1520,17 @@ int add_mirror_images(struct cmd_context *cmd, struct logical_volume *lv,
 	return 1;
 
   out_remove_log:
-	if (log_lv && (!lv_remove(log_lv) || !vg_write(log_lv->vg) ||
-		       (backup(log_lv->vg), !vg_commit(log_lv->vg))))
-		log_error("Manual intervention may be required to remove "
-			  "abandoned log LV before retrying.");
-
-  out_remove_imgs:
+	if (log_lv) {
+		if (!lv_remove(log_lv) ||
+		    !vg_write(log_lv->vg) ||
+		    !vg_commit(log_lv->vg))
+			log_error("Manual intervention may be required to remove "
+				  "abandoned log LV before retrying.");
+		else
+			backup(log_lv->vg);
+	}
+  out_remove_images:
+	alloc_destroy(ah);
 	return 0;
 }
 
@@ -1458,7 +1542,7 @@ int add_mirror_images(struct cmd_context *cmd, struct logical_volume *lv,
 int lv_add_mirrors(struct cmd_context *cmd, struct logical_volume *lv,
 		   uint32_t mirrors, uint32_t stripes,
 		   uint32_t region_size, uint32_t log_count,
-		   struct list *pvs, alloc_policy_t alloc, uint32_t flags)
+		   struct dm_list *pvs, alloc_policy_t alloc, uint32_t flags)
 {
 	if (!mirrors && !log_count) {
 		log_error("No conversion is requested");
@@ -1509,7 +1593,7 @@ int lv_add_mirrors(struct cmd_context *cmd, struct logical_volume *lv,
  */
 int lv_remove_mirrors(struct cmd_context *cmd __attribute((unused)),
 		      struct logical_volume *lv,
-		      uint32_t mirrors, uint32_t log_count, struct list *pvs,
+		      uint32_t mirrors, uint32_t log_count, struct dm_list *pvs,
 		      uint32_t status_mask)
 {
 	uint32_t new_mirrors;
