@@ -1,14 +1,14 @@
 /*
  * Copyright (C) 2001-2004 Sistina Software, Inc. All rights reserved.
- * Copyright (C) 2004 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2004-2007 Red Hat, Inc. All rights reserved.
  *
  * This file is part of LVM2.
  *
  * This copyrighted material is made available to anyone wishing to use,
  * modify, copy, or redistribute it subject to the terms and conditions
- * of the GNU General Public License v.2.
+ * of the GNU Lesser General Public License v.2.1.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
@@ -31,7 +31,7 @@ static void _pvscan_display_single(struct cmd_context *cmd,
 
 	/* short listing? */
 	if (arg_count(cmd, short_ARG) > 0) {
-		log_print("%s", dev_name(get_pv_dev(pv)));
+		log_print("%s", pv_dev_name(pv));
 		return;
 	}
 
@@ -48,7 +48,7 @@ static void _pvscan_display_single(struct cmd_context *cmd,
 
 	memset(pv_tmp_name, 0, sizeof(pv_tmp_name));
 
-	vg_name_len = strlen(get_pv_vg_name(pv)) + 1;
+	vg_name_len = strlen(pv_vg_name(pv)) + 1;
 
 	if (arg_count(cmd, uuid_ARG)) {
 		if (!id_write_format(&pv->id, uuid, sizeof(uuid))) {
@@ -57,43 +57,43 @@ static void _pvscan_display_single(struct cmd_context *cmd,
 		}
 
 		sprintf(pv_tmp_name, "%-*s with UUID %s",
-			pv_max_name_len - 2, dev_name(get_pv_dev(pv)), uuid);
+			pv_max_name_len - 2, pv_dev_name(pv), uuid);
 	} else {
-		sprintf(pv_tmp_name, "%s", dev_name(get_pv_dev(pv)));
+		sprintf(pv_tmp_name, "%s", pv_dev_name(pv));
 	}
 
-	if (!*get_pv_vg_name(pv)) {
+	if (is_orphan(pv)) {
 		log_print("PV %-*s    %-*s %s [%s]",
 			  pv_max_name_len, pv_tmp_name,
 			  vg_max_name_len, " ",
 			  pv->fmt ? pv->fmt->name : "    ",
-			  display_size(cmd, get_pv_size(pv)));
+			  display_size(cmd, pv_size(pv)));
 		return;
 	}
 
-	if (get_pv_status(pv) & EXPORTED_VG) {
-		strncpy(vg_name_this, get_pv_vg_name(pv), vg_name_len);
+	if (pv_status(pv) & EXPORTED_VG) {
+		strncpy(vg_name_this, pv_vg_name(pv), vg_name_len);
 		log_print("PV %-*s  is in exported VG %s "
 			  "[%s / %s free]",
 			  pv_max_name_len, pv_tmp_name,
 			  vg_name_this,
-			  display_size(cmd, (uint64_t) get_pv_pe_count(pv) *
-				       get_pv_pe_size(pv)),
-			  display_size(cmd, (uint64_t) (get_pv_pe_count(pv) -
-						get_pv_pe_alloc_count(pv))
-				       * get_pv_pe_size(pv)));
+			  display_size(cmd, (uint64_t) pv_pe_count(pv) *
+				       pv_pe_size(pv)),
+			  display_size(cmd, (uint64_t) (pv_pe_count(pv) -
+						pv_pe_alloc_count(pv))
+				       * pv_pe_size(pv)));
 		return;
 	}
 
-	sprintf(vg_tmp_name, "%s", get_pv_vg_name(pv));
+	sprintf(vg_tmp_name, "%s", pv_vg_name(pv));
 	log_print("PV %-*s VG %-*s %s [%s / %s free]", pv_max_name_len,
 		  pv_tmp_name, vg_max_name_len, vg_tmp_name,
 		  pv->fmt ? pv->fmt->name : "    ",
-		  display_size(cmd, (uint64_t) get_pv_pe_count(pv) * 
-					       get_pv_pe_size(pv)),
-		  display_size(cmd, (uint64_t) (get_pv_pe_count(pv) - 
-						get_pv_pe_alloc_count(pv)) *
-					   get_pv_pe_size(pv)));
+		  display_size(cmd, (uint64_t) pv_pe_count(pv) *
+					       pv_pe_size(pv)),
+		  display_size(cmd, (uint64_t) (pv_pe_count(pv) -
+						pv_pe_alloc_count(pv)) *
+					   pv_pe_size(pv)));
 	return;
 }
 
@@ -103,7 +103,7 @@ int pvscan(struct cmd_context *cmd, int argc __attribute((unused)),
 	int new_pvs_found = 0;
 	int pvs_found = 0;
 
-	struct list *pvslist;
+	struct dm_list *pvslist;
 	struct pv_list *pvl;
 	struct physical_volume *pv;
 
@@ -120,25 +120,33 @@ int pvscan(struct cmd_context *cmd, int argc __attribute((unused)),
 	}
 
 	if (arg_count(cmd, exported_ARG) || arg_count(cmd, novolumegroup_ARG))
-		log_print("WARNING: only considering physical volumes %s",
+		log_warn("WARNING: only considering physical volumes %s",
 			  arg_count(cmd, exported_ARG) ?
 			  "of exported volume group(s)" : "in no volume group");
 
+	if (!lock_vol(cmd, VG_GLOBAL, LCK_VG_WRITE)) {
+		log_error("Unable to obtain global lock.");
+		return ECMD_FAILED;
+	}
+
 	persistent_filter_wipe(cmd->filter);
-	lvmcache_destroy();
+	lvmcache_destroy(cmd, 1);
 
 	log_verbose("Walking through all physical volumes");
-	if (!(pvslist = get_pvs(cmd)))
+	if (!(pvslist = get_pvs(cmd))) {
+		unlock_vg(cmd, VG_GLOBAL);
+		stack;
 		return ECMD_FAILED;
+	}
 
 	/* eliminate exported/new if required */
-	list_iterate_items(pvl, pvslist) {
+	dm_list_iterate_items(pvl, pvslist) {
 		pv = pvl->pv;
 
 		if ((arg_count(cmd, exported_ARG)
-		     && !(get_pv_status(pv) & EXPORTED_VG))
-		    || (arg_count(cmd, novolumegroup_ARG) && (*get_pv_vg_name(pv)))) {
-			list_del(&pvl->list);
+		     && !(pv_status(pv) & EXPORTED_VG))
+		    || (arg_count(cmd, novolumegroup_ARG) && (!is_orphan(pv)))) {
+			dm_list_del(&pvl->list);
 			continue;
 		}
 
@@ -154,33 +162,34 @@ int pvscan(struct cmd_context *cmd, int argc __attribute((unused)),
 ********/
 		pvs_found++;
 
-		if (!*get_pv_vg_name(pv)) {
+		if (is_orphan(pv)) {
 			new_pvs_found++;
-			size_new += get_pv_size(pv);
-			size_total += get_pv_size(pv);
+			size_new += pv_size(pv);
+			size_total += pv_size(pv);
 		} else
-			size_total += get_pv_pe_count(pv) * get_pv_pe_size(pv);
+			size_total += pv_pe_count(pv) * pv_pe_size(pv);
 	}
 
 	/* find maximum pv name length */
 	pv_max_name_len = vg_max_name_len = 0;
-	list_iterate_items(pvl, pvslist) {
+	dm_list_iterate_items(pvl, pvslist) {
 		pv = pvl->pv;
-		len = strlen(dev_name(get_pv_dev(pv)));
+		len = strlen(pv_dev_name(pv));
 		if (pv_max_name_len < len)
 			pv_max_name_len = len;
-		len = strlen(get_pv_vg_name(pv));
+		len = strlen(pv_vg_name(pv));
 		if (vg_max_name_len < len)
 			vg_max_name_len = len;
 	}
 	pv_max_name_len += 2;
 	vg_max_name_len += 2;
 
-	list_iterate_items(pvl, pvslist)
+	dm_list_iterate_items(pvl, pvslist)
 	    _pvscan_display_single(cmd, pvl->pv, NULL);
 
 	if (!pvs_found) {
 		log_print("No matching physical volumes found");
+		unlock_vg(cmd, VG_GLOBAL);
 		return ECMD_PROCESSED;
 	}
 
@@ -191,5 +200,7 @@ int pvscan(struct cmd_context *cmd, int argc __attribute((unused)),
 		  display_size(cmd, (size_total - size_new)),
 		  new_pvs_found, display_size(cmd, size_new));
 
+	unlock_vg(cmd, VG_GLOBAL);
+
 	return ECMD_PROCESSED;
 }
